unit InterbaseIntf;

interface

uses Windows;

{---------------------------------------------------------
  Each Plugin have to export three function:
  get_plugin_info - getting information about plugin,
    menu items captions, edited object's type etc.,
  plugin_options_execute - options dialog for Plugin (if needed)
  plugin_execute - launch Plugin

  More sopthisticated Plugins can include
  three additional function. They allow Plugin to be included
  as pages TabControl (rather then Modal dialog).
  These functions have to named as
  plugin_create_controls - called during creating object editor
  plugin_destroy_controls - called to destry object editor
  plugin_refresh_controls - called when user activate page
    which was created by Plugin.
 ---------------------------------------------------------}

const
  { IB Fields Types }
  iftSmallInt  = 7;
  iftInteger   = 8;
  iftQuad      = 9;
  iftFloat     = 10;
  iftD_Float   = 11;
  iftDate      = 12;  // IB 6.0
  iftTime      = 13;  // IB 6.0
  iftChar      = 14;
  iftInt64     = 16;  // IB 6.0
  iftBoolean   = 17;  // IB 7.0
  iftDouble    = 27;
  iftDateTime  = 35;
  iftVarChar   = 37;
  iftCString   = 40;
  iftBlob      = 261;

type

  { Execution information }
  PplExecInfo = ^TplExecInfo;
  TplExecInfo = record
    DBIndex: Integer;
    ObjectName: PChar;
    ObjectEditor: pointer; // TForm ascendent
    Reserved: pointer;
  end;

  TIBObjectEditor = (oeNone, oeDomain, oeTable, oeView, oeProcedure,
      oeTrigger, oeGenerator, oeException, oeFunction,
      oeTableColumn, oeTableIndex, oeTableConstraint, oeBlobFilter);
  TIBObjectEditors = set of TIBObjectEditor;

  { Plugin information }
  PIBManagerPluginInfo = ^TIBManagerPluginInfo;
  TIBManagerPluginInfo = record
    PluginName, Description: PChar;
    IsCommonTool: boolean;
    { Is Plugin common tool or serve to control specific objects}
    PluginObjectEditors: TIBObjectEditors;
    { Processed objects' types }
    MenuCaption: PChar;
    MenuShortCut: word;
    OptionsMenuCaption: PChar;
    OptionsMenuShortCut: word;
    PluginKind: word;
    { If IsCommonTool = false, (Plugin is object editor) then
      this parameter specify kind of editor appearance
      0 - menu item
      1 - add tab to object editor. In this case MenuCaption
          will represent tab caption
    }

    HasOptions: boolean;
    { Specified has Plugin options dialog or not }
    PlaceOnToolbar: boolean;
    { Put button on Toolbar or not }
    UnloadAfterExecute: boolean;
    { Unload plugin after execute or not }
  end;

  { IBManager objects lists }
  TIBITablesList = Pointer;
  TIBITableFields = Pointer;
  TIBIQuery = Pointer;
  TIBITransaction = Pointer;

  { Information about table }
  TIBITableInfo = record
    TableName: PChar;
    TableType: WORD;
    Comment: PChar;
    AutoIncrement: integer;
    AvgRowLength: integer;
    CheckSum: boolean;
    DelayKeyWrite: boolean;
    MaxRows: integer;
    MinRows: integer;
    PackKeys: boolean;
    RowFormat: WORD;
    Union: PChar;
    TypeAsString: PChar;
    Description: PChar;
  end;

  { Information about field of table }
  TIBIFieldInfo = record
    FieldName: PChar;
    FieldPos: WORD;
    FieldType: WORD;
    FieldSize: WORD;
    FieldPrecision: integer; // <= 0
    DefValue: PChar;
    { Field Flags }
    NotNull: boolean;
    AutoIncrement: boolean;
    Binary: boolean;
    Unsigned: boolean;
    ZeroFill: boolean;
    PrimaryKey: boolean;
    Unique: boolean;
    FieldDomain     : PChar;
    FieldSubType    : integer; // for Blob fields
    FieldSegmentLen : WORD;    // for Blob fields;
    FieldLen        : WORD;
    FieldScale      : integer; // <= 0
    CharacterLen    : integer;
    Dimensions      : PWideChar;   // In form [a,b,c]
    DefaultSource   : PWideChar;
    ComputedSource  : PWideChar;
    CollationId     : integer;
    CharSetId       : integer;
    Collation       : PChar;
    CharSet         : PWideChar;
    FieldTypeAsString : PWideChar;
  end;


  { Database functions }
  TDatabasesCount = function : integer;
  TDatabaseAlias = function(DBIndex : integer) : PChar;
  TDatabaseName = function(DBIndex : integer) : PChar;
  TDatabaseActive = function(DBIndex : integer) : boolean;
  TOpenDatabase = function(DBIndex : integer) : integer;
  TCloseDatabase = function(DBIndex : integer) : integer;
  TFormatIdentifier = function(Identifier: PChar; DBIndex: integer): PChar;

  { Table and fields functions }
  TGetTables = function(DBIndex : integer; var TablesCount : integer) : TIBITablesList;
  TFetchTableInfo = function(Tables : TIBITablesList; Index : Integer; var Info : TIBITableInfo) : integer;
  TFreeTables = procedure(Tables : TIBITablesList);
  TFieldsOfTable = function(DBIndex : integer; TableName : PChar; var FieldsCount : integer) : TIBITableFields;
  TFieldInfo = function(Fields : TIBITableFields; Index : integer; var Info : TIBIFieldInfo) : integer;
  TFreeTableFields = procedure(Fields : TIBITableFields);

  { Transaction functions }
  TCreateTransaction = function(DBIndex : integer) : TIBITransaction;
  TStartTransaction = procedure(Transaction : TIBITransaction);
  TCommitTransaction = procedure(Transaction : TIBITransaction);
  TRollbackTransaction = procedure(Transaction : TIBITransaction);
  TFreeTransaction = procedure(Transaction : TIBITransaction);

  { Query functions }
  TCreateQuery = function(DBIndex : integer; Transaction : TIBITransaction) : TIBIQuery;
  TPrepareQuery = procedure(Query : TIBIQuery);
  TSetQuerySQL = procedure(Query : TIBIQuery; SQL : PChar);
  TExecQuery = function(Query : TIBIQuery): boolean;
  TParamCheck = procedure(Query : TIBIQuery; ParamCheck : boolean);
  TSetParam = procedure(Query : TIBIQuery; ParamIndex : integer; ParamValue : Variant);
  TSetTypedParam = procedure(Query : TIBIQuery; ParamIndex, ParamType : integer; ParamValue : Variant);
  TGetFieldByName = function(Query : TIBIQuery; FieldName : PChar) : variant;
  TGetFieldByIndex = function(Query : TIBIQuery; FieldIndex : integer) : variant;
  TQueryEof = function(Query : TIBIQuery) : boolean;
  TQueryNext = procedure(Query : TIBIQuery);
  TQueryFirst = procedure(Query : TIBIQuery);
  TQueryLast = procedure(Query : TIBIQuery);
  TQueryPrior = procedure(Query : TIBIQuery);
  TFreeQuery = procedure(Query : TIBIQuery);
  TCloseQuery = procedure(Query : TIBIQuery);
  TClearParams = procedure(Query : TIBIQuery);

  { Controls and Forms functions }
  TCreateChildForm = function (FormCaption: PChar): pointer; // TForm
  { Create and get handle of form that will controlled by
  WindowManager. Not implemented }

  TGetComponentByName = function (Form: pointer; Name: PChar): pointer;

  TErrorMessage = procedure (Caption: PChar);
  TInfoMessage = procedure (Caption: PChar);

  { Plugin interface }
  TIBManagerInterface = record
    IBManagerApplication: pointer;
    DatabasesCount : TDatabasesCount;
    DatabaseAlias : TDatabaseAlias;
    DatabaseName : TDatabaseName;
    DatabaseActive : TDatabaseActive;
    FormatIBIdentifier: TFormatIdentifier;

    OpenDatabase : TOpenDatabase;
    CloseDatabase : TCloseDatabase;

    GetTables : TGetTables;
    FetchTableInfo : TFetchTableInfo;
    FreeTables : TFreeTables;

    FieldsOfTable : TFieldsOfTable;
    FieldInfo : TFieldInfo;
    FreeTableFields : TFreeTableFields;

    CreateTransaction : TCreateTransaction;
    StartTransaction : TStartTransaction;
    CommitTransaction : TCommitTransaction;
    RollbackTransaction : TRollbackTransaction;
    FreeTransaction : TFreeTransaction;

    CreateQuery : TCreateQuery;
    PrepareQuery: TPrepareQuery;
    SetQuerySQL : TSetQuerySQL;
    ExecQuery : TExecQuery;
    ParamCheck : TParamCheck;
    SetParam : TSetParam;
    SetTypedParam: TSetTypedParam;
    GetFieldByName : TGetFieldByName;
    GetFieldByIndex : TGetFieldByIndex;
    QueryEof : TQueryEof;
    QueryNext : TQueryNext;
    QueryFirst : TQueryFirst;
    QueryLast : TQueryLast;
    QueryPrior : TQueryPrior;
    FreeQuery : TFreeQuery;
    CloseQuery: TCloseQuery;

    ErrorMessage: TErrorMessage;
    InfoMessage: TInfoMessage;
  end;

  TGetPluginInfoProc = procedure(PluginInfo: PIBManagerPluginInfo); stdcall;
  TPluginOptionsExecuteProc = procedure(Intf : TIBManagerInterface); stdcall;
  TPluginExecuteProc = procedure(ExecInfo: PplExecInfo; Intf : TIBManagerInterface); stdcall;

  TPluginCreateControlsProc = procedure(ExecInfo: PplExecInfo; Intf : TIBManagerInterface; Parent: pointer); stdcall;
  TPluginDestroyControlsProc = procedure(ExecInfo: PplExecInfo; Intf : TIBManagerInterface; Parent: pointer); stdcall;
  TPluginRefreshControlsProc = procedure(ExecInfo: PplExecInfo; Intf : TIBManagerInterface; Parent: pointer); stdcall;

implementation

end.
